package com.sample;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import junit.framework.TestCase;

import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.db.SchedulerSession;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;
import org.jbpm.scheduler.impl.Scheduler;
import org.jbpm.taskmgmt.exe.TaskInstance;

/**
 * An use case showing a set of user tasks competing against each other and a deadline.
 * @author Alejandro Guizar
 * @version $Revision$ $Date$
 */
public class DeferredChoiceTest extends TestCase {
  
  JbpmConfiguration jbpmConfiguration = JbpmConfiguration.getInstance();
  JbpmContext jbpmContext;
  
  Scheduler scheduler = new Scheduler();

	protected void setUp() throws Exception {
    // create the db schema
    jbpmConfiguration.createSchema();
    // create a context
    jbpmContext = jbpmConfiguration.createJbpmContext();
    // parse and deploy the process definition
    ProcessDefinition processDefinition = ProcessDefinition.parseXmlResource("deferred.choice.par/processdefinition.xml");
    jbpmContext.deployProcessDefinition(processDefinition);
    // configure and start the scheduler
    scheduler.start();
  }

  protected void tearDown() throws Exception {
    // stop the scheduler
    scheduler.stop();
    // close the context
    jbpmContext.close();
    // drop the schema
    jbpmConfiguration.dropSchema();
  }

  /**
   * Validates the scenario where a user completes one task; the process must cancel the 
   * remaining tasks and cancel the timer.
   */
  public void testTaskMutualCancellation() {
    // instantiate the process
    ProcessInstance processInstance = jbpmContext.newProcessInstanceForUpdate("Deferred Choice");
    // move to the task node
    processInstance.signal();
    
    // get the generated timers
    Token rootToken = processInstance.getRootToken();
    SchedulerSession schedulerSession = jbpmContext.getSchedulerSession();
    List timers = schedulerSession.findTimersByName("deadline", rootToken);
    assertEquals(1, timers.size());
    
    // get the generated task instances
    Collection tasks = processInstance.getTaskMgmtInstance().getTaskInstances();
    // get one of the tasks
    Iterator taskIt = tasks.iterator();
    TaskInstance openTask = (TaskInstance) taskIt.next();
    // ensure task is open
    assertTrue(openTask.isOpen());
    
    // end the open task
    openTask.end();
    
    // ensure the remaining tasks are cancelled (cancellation occurs upon leaving the node)
    while (taskIt.hasNext()) {
      TaskInstance cancelledTask = (TaskInstance) taskIt.next();
      assertFalse(cancelledTask.isOpen());
    }
    // ensure the timer is cancelled
    timers = schedulerSession.findTimersByName("deadline", rootToken);
    assertEquals(0, timers.size());
    
    // check the process had a happy ending
    assertEquals("happyend", rootToken.getNode().getName());
    assertTrue(processInstance.hasEnded());
	}
  
  /**
   * Validates the scenario where users take too long; the process must cancel all tasks.  
   */
  public void testTaskTimeout() {
    // instantiate the process
    ProcessInstance processInstance = jbpmContext.newProcessInstanceForUpdate("Deferred Choice");
    // move to the task node
    processInstance.signal();  
    // save and close
    jbpmContext.close();
    
    // wait long enough for the timer to cancel the tasks
    try {
      Thread.sleep(11000);
    }
    catch (InterruptedException e) {
      fail("got interrupted, tasks might not be cancelled yet");
    }
    
    // reload
    jbpmContext = jbpmConfiguration.createJbpmContext();
    processInstance = jbpmContext.loadProcessInstance(processInstance.getId());
    Collection tasks = processInstance.getTaskMgmtInstance().getTaskInstances();
    // ensure all task instances are cancelled
    Iterator taskIt = tasks.iterator();
    while (taskIt.hasNext()) {
      TaskInstance cancelledTask = (TaskInstance) taskIt.next();
      assertFalse(cancelledTask.isOpen());
    }
    // check the process had a bitter ending
    assertEquals("bitterend", processInstance.getRootToken().getNode().getName());
    assertTrue(processInstance.hasEnded());
  }
}
